<?php

namespace DiviBooster\PhpCode;

class PhpCode {

    private $code;

    static function fromString($code) {
        return new self($code);
    }

    private function __construct($code) {
        $this->code = (string) $code;
    }

    public function output($allow_errors = false) {
        $code = $this->code;
        $this->error_clear_last();
        ob_start();

        // Convert notices into exceptions
        $previousErrorHandler = set_error_handler(function ($severity, $message, $file, $line) {
            throw new \Exception($message . " on line " . $line, $severity);
        }, E_ALL);

        $error = null;
        try {
            if ($allow_errors) {
                eval($code);
            } else {
                @eval($code);
            }
        } catch (\ParseError $err) {
            $error = $err->getMessage() . " on line " . $err->getLine();
        } catch (\Exception $err) {
            $error = $err->getMessage();
            // Add stack trace, if callable
            if (method_exists($err, 'getTraceAsString')) {
                $stacktrace = $err->getTraceAsString();

                // Iterate through the stack trace and remove all lines after the one ending : eval()
                $lines = explode("\n", $stacktrace);
                $new_lines = array();
                foreach ($lines as $line) {
                    if (strpos($line, 'eval()') === false) {
                        continue;
                    }
                    $new_lines[] = $line;
                }
                $stacktrace = implode("\n", $new_lines);

                $error .= "\n\nStacktrace:\n" . $stacktrace;
            }
        } finally {
            // Restore normal error handling
            set_error_handler($previousErrorHandler);
        }
        $result = ob_get_clean();

        if ($error) {
            throw new \Exception($error);
        }

        $last_error = error_get_last();
        if (!empty($last_error['message'])) {
            throw new \Exception($last_error['message']);
        }
        return $result;
    }

    // Backwards compatible version of PHP's error_clear_last()
    private function error_clear_last() {
        if (function_exists('error_clear_last')) { // PHP ^7.0
            error_clear_last();
        } else {
            @trigger_error("");
        }
    }

    public function __toString() {
        return (string) $this->code;
    }
}
